package com.ruoyi.adsdesk.business.operation;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.adsdesk.param.SimulateLoginParam;
import com.ruoyi.common.constant.StrConstants;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.security.LoginUser;
import org.apache.logging.log4j.util.Strings;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.net.URI;

/**
 * 模拟登录业务
 * @author Xingbz
 * 2022-08-12
 */
@Service
public class AdsSimulateLoginBusiness {

    public String querySimulateLoginAuthToken(SimulateLoginParam loginParam) {
        RestTemplate restTemplate = new RestTemplate();

        JSONObject param = new JSONObject();
        param.put("secret","20230000");
        param.put("userId",loginParam.getUserId());
        param.put("userName",loginParam.getUserName());

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        HttpEntity<?> requestEntity = new HttpEntity<>(param, headers);

        String apiUrlPre = StringUtils.equals(loginParam.getOutEnv(), "test2") ? "test-api2" : "pre-api";
        String urlPre = "http://" + apiUrlPre + loginParam.getOutApp() + "/";

        String url = urlPre + "adsdesk/api/open_api/system/authorize";
        String responseStr = restTemplate.exchange(URI.create(url), HttpMethod.POST, requestEntity, String.class).getBody();
        responseStr = clearResponseStr(responseStr);

        if(Strings.isEmpty(responseStr)){
            return null;
        }
        JSONObject jsonObject = JSON.parseObject(responseStr);
        int code = (int) jsonObject.get("code");
        JSONArray data = (JSONArray) jsonObject.get("data");
        JSONObject first = (JSONObject) data.get(0);
        String token = (String) first.get("value");

        LoginUser loginUser = SecurityUtils.getLoginUserIgnoreUnAuth();
        JSONObject loginJsonObject = new JSONObject();
        loginJsonObject.put("user",loginUser.getUsername());
        loginJsonObject.put("uniqueCode","c_"+System.currentTimeMillis());
        loginJsonObject.put("token",token);
        return "http://" + loginParam.getOutEnv() + loginParam.getOutApp() + "?authToken=" + loginJsonObject;
    }

    private String clearResponseStr(String responseStr) {
        if (ObjectUtil.isNull(responseStr)) {
            return null;
        }

        // 双引号包括的字符串
        if (StrUtil.startWith(responseStr, StrConstants.DOUBLE_QUOTES) && StrUtil.endWith(responseStr, StrConstants.DOUBLE_QUOTES)) {
            int placeholderCount = StrUtil.count(responseStr, "\\\"");
            //  类似于"{\"en\":\"zhangsan\",\"cn\":\"张三\"}"转义之后的json字符串
            if (placeholderCount > 0 && placeholderCount % 2 == 0) {
                return StrUtil.replace(StrUtil.removeSuffix(StrUtil.removePrefix(responseStr, StrConstants.DOUBLE_QUOTES), StrConstants.DOUBLE_QUOTES), "\\\"", StrConstants.DOUBLE_QUOTES);
            }
        }

        return responseStr;
    }
}
