package com.ruoyi.adsdesk.util;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.adsdesk.param.AdsListBaseParam;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.page.TableDataInfo;

import java.util.List;
import java.util.function.BiFunction;

import static java.util.stream.Collectors.toList;

/**
 * Ads分页相关工具
 * @author Xingbz
 * 2022-6-7
 */
public interface AdsPageUtil {

    static <T> Page<T> buildPage(AdsListBaseParam baseParam) {
        return new Page<T>(baseParam.getPageNum(), baseParam.getPageSize());
    }

    static <T, R> IPage<R> convert(IPage<T> page, List<R> rList) {
        return new Page<R>(page.getCurrent(), page.getSize(), page.getTotal()).setRecords(rList);
    }

    /**
     * IPage 的泛型转换
     *
     * @param mapper 转换函数
     * @param <R>    转换后的泛型
     * @return 转换泛型后的 IPage
     */
    @SuppressWarnings("unchecked")
    static <T, U, R> IPage<R> convert(IPage<T> page, U u, BiFunction<? super T, U, ? extends R> mapper) {
        List<R> collect = page.getRecords().stream().map(item -> mapper.apply(item, u)).collect(toList());
        return ((IPage<R>) page).setRecords(collect);
    }

    /**
     * 根据全量集合进行内存分页, 适用于API端没有分页的数据
     * @param list 全量数据集合(如果频繁使用一定要缓存)
     * @param current 当前页数
     * @param size 每页大小
     * @return 分页结果
     */
    static <T> Page<T> pageByList(List<T> list, Integer current, Integer size) {
        int total = list.size();
        Page<T> page = new Page<>(current, size, total);
        // 如果仅有一页或一页不到的数据, 但是查询了第二页之后的 , 直接返回空数据(但是要附上总记录数)
        if (total <= size && current > 1) {
            return page;
        }

        // 如果有一页以上的数据, 则根据前端的分页参数计算本页数据
        if (total > size) {
            int fromIndex = (current - 1) * size;
            // 如果仅有100条 , 但查询了11页(从101开始), 则直接返回空数据(附上总记录数)
            if (fromIndex > total) {
                return page;
            }
            int toIndex = fromIndex + size;
            // Math.min 是为了预防记录数超出时(最后一页时记录数不满一页)报异常
            list = list.subList(fromIndex, Math.min(toIndex, total));
        }

        page.setRecords(list);
        return page;
    }

    static TableDataInfo transformToTableData(IPage<?> pageResult) {
        TableDataInfo tableDataInfo = new TableDataInfo();
        tableDataInfo.setCode(HttpStatus.SUCCESS);
        tableDataInfo.setMsg("查询成功");
        tableDataInfo.setRows(pageResult.getRecords());
        tableDataInfo.setTotal(pageResult.getTotal());
        return tableDataInfo;
    }
}
